#!/usr/local/bin/perl
package main;
#use strict;

###############################################################################
#							     FC_REMIND.PL
#
# Version: 1.0.0
#
# Last Modified:  25 APR 1999
#
# This program was written by Michael E. Schechter (mschechter@earthlink.net)
# to provide a utility to the FrameCal Perl script to send out reminder
# messages of events.  This program only works with FrameCal's format for
# storing information.
#
# Note that this program DOES NOT run from the web.  Instead, the user must
# have access to a scheduling utility on the web server, such as cron.  The
# script should be entered into the scheduler, and given an argument of the
# calendar to work with, as in
# '/usr/local/www/cgi-bin/framecal/fc_remind.pl demo', where demo is the name
# of the directory containing the .setup files, etc., for a calendar.  Of
# course, if you are using just one calendar, you can hardcode it into the
# script, if you would like.  Also, note that since this does not run from the
# web, or from a web server, modules such as mod_perl, fastCGI, and perlEx will
# not have any impact on the script's performance.
#
# This script requires the mail library for your OS, which should already
# be installed for FrameCal.
#
# Also, please note that this program is really just pulling together a few of
# the subroutines from framecal.pl, and thus, if you modify any of these
# subroutines in framecal.pl, you must also make modifications here.
#
# You may copy this under the terms of the GNU General Public License.
###############################################################################

# First, we need to set up our script with some general information regarding
# file locations, etc.  We hard code in our absolute path to the cgi-bin
# directory, or the directory directly above the /framecal directory for the
# installation.  We then get the location of the database files that we will
# be working with by getting the argument passed to the script, and defining
# the libs directory for use when we need the mail-lib.pl file.

my( $abs_path ) = "/usr/home/www/cgi-bin";
my( $file_loc ) = shift( @ARGV );
$file_loc = "framecal/$file_loc";
my( $lib ) = "libs";

# The @num_days array contains the number of days you would like reminder
# messages sent out.  The default value is 1 and 7, meaning that for each
# event, an email will be sent out 1 day and 7 days prior to the event
# to remind people about it.  Note that only integer values may be used in
# this array.

my( @num_days ) = ( 1, 7 );

# Let's declare the rest of our variables for the 'main' section of the
# program.

my( $lt_jd, $x, $nd_y, $nd_m, $nd_d, $file, $day, $time, $user, $cat, $id,
	$tz, $dur_h, $dur_m, $name, $email, $notified, $linkurl, $linktitle,
	$loc, $subject, $body, $date );

# We need to see if the calendar we are looking at is in maintenance, in
# which case we can not do anything.

unless( -e "$abs_path/$file_loc/fc_main.txt" ) {

# We need some information from the calendar.setup file, so let's go ahead
# and require it into the script now, as well as our mail library.

	require( "$abs_path/$file_loc/calendar.setup" )
		|| die "$0: Unable to open calendar.setup file for $file_loc. $!\n";
	require( "$abs_path/$lib/mail-lib.pl" )
		|| die "$0: Unable to open mail library. $!\n";

# We need to know what day it is right now, so we can calculate which days
# we need to send emails out for.  We get our date information here just
# like we did in framecal.pl.  We then calculate the julian day of the
# current date, as that allows us to do very simple arithmetic to find out
# the dates that we will be sending out reminders for.

	my( $lt_d, $lt_m, $lt_y ) = (localtime(time))[3,4,5];
	$lt_m += 1;
	$lt_y += 1900;
	$lt_jd = &julian_date( $lt_y, $lt_m, $lt_d );

# We now start a loop for each day in our @num_days array.  We will calculate the
# date from today for each value in the array, by adding the number to our
# julian day for today, then back calculating the new date using the jdn_to_date
# subroutine.  We use the year and month to determine which database file to look
# in, then, if the file exists and is not zero size, we open it up, and read
# through it.

	for( $x = 0; $x < scalar( @num_days ); $x++ ) {
		( $nd_y, $nd_m, $nd_d ) = &jdn_to_date( $lt_jd + $num_days[ $x ] );
		$file = "$nd_y".sprintf( "%.2d", $nd_m ).".events";
		if( -s "$abs_path/$file_loc/$file" ) {
			open( FILE, "$abs_path/$file_loc/$file" );

# As we read through the file, we split up each line.  If the day is equal to
# the day we calculated above, we have found a match.  We get the information
# on the event using the get_event_info subroutine, concatenate our email
# addresses into the $notified string, and create our $date string.  If the
# $notified string is not empty, we email everyone in the string the info
# on the event using the notification subroutine.

# Once we get to a day value in our database file that is greater than the day
# we calculated, we stop reading through it.

			while( <FILE> ) {
				( $day, $time, $user, $cat, $id ) = split( /\|/, $_ );
				if( $day == $nd_d ) {
					chomp( $id );
					( $time, $tz, $dur_h, $dur_m, $name, $email, $notified, $linkurl,
					  $linktitle, $loc, $subject, $body ) = &get_event_info( "$id" );
					$notified = "$email,$notified";
					$date = "$nd_m/$nd_d/$nd_y";
					if( $notified ne '' ) {
						&notification( $date, $time, $tz, $dur_h, $dur_m, $notified,
									   $loc, $subject, $body, $linkurl, $linktitle,
									   $cat, $email, $name );
					}
				} elsif( $day > $nd_d ) {
					last;
				}
			}

# We are now done reading, so we close our database file.

			close( FILE );
		}
	}
}

# We are done with our for loop, and the program exits.  From here on out, we
# just have subroutines.

# SUB JULIAN_DATE #

# See comments in the fc_recur.pl file.

sub julian_date {
	my( $year, $month, $day ) = @_;
	my( $a, $b, $c, $jdn ) = ();
	$a = int((14 - $month)/12);
	$b = $year + 4800 - $a;
	$c = $month + (12 * $a) - 3;
	$jdn = $day+int((153*$c+2)/5)+(365*$b)+int($b/4)-int($b/100)+int($b/400)-32045;
	return( $jdn );
}

# SUB JDN_TO_DATE #

# See comments in the fc_recur.pl file.

sub jdn_to_date {
	my( $jdn ) = @_;
	my( $a, $b, $c, $d, $e, $m, $year, $month, $day ) = ();
	$a = $jdn + 32045;
	$b = int( (4*($a+36524))/146097 ) - 1;
	$c = $a - int( (146097*$b)/4 );
	$d = int( (4*($c+365))/1461 ) - 1;
	$e = $c - int( (1461*$d)/4 );
	$m = int( (5*($e-1)+2)/153 );
	$day = $e - int( (153*$m+2)/5 );
	$month = $m + 3 - (12 * int($m/10));
	$year = (100*$b) + $d - 4800 + int($m/10);
	return( $year, $month, $day );
}

# SUB GET_EVENT_INFO #

# See comments in the framecal.pl file.

sub get_event_info {
	my( $filename ) = $_[0];
	my( @event_array );
	open( EVENTFILE, "$abs_path/$file_loc/events/$filename" );
	while( <EVENTFILE> ) {
		   if( /^time->(.*)/io )		 { push( @event_array, $1 ); }
		elsif( /^timezone->(.*)/io )	 { push( @event_array, $1 ); }
		elsif( /^durationhrs->(.*)/io )  { push( @event_array, $1 ); }
		elsif( /^durationmins->(.*)/io ) { push( @event_array, $1 ); }
		elsif( /^name->(.*)/io )		 { push( @event_array, $1 ); }
		elsif( /^email->(.*)/io )		 { push( @event_array, $1 ); }
		elsif( /^notified->(.*)/io )	 { push( @event_array, $1 ); }
		elsif( /^linkurl->(.*)/io )		 { push( @event_array, $1 ); }
		elsif( /^linktitle->(.*)/io )	 { push( @event_array, $1 ); }
		elsif( /^location->(.*)/io )	 { push( @event_array, $1 ); }
		elsif( /^subject->(.*)/io )		 { push( @event_array, $1 ); }
		elsif( /^body->(.*)/io )		 { push( @event_array, $1 ); }
	}
	close( EVENTFILE );
	chomp( @event_array );
	return( @event_array );
}

# SUB GET_END_TIME #

# See comments in the framecal.pl file.

sub get_end_time {
	my( $time,$dur_hrs,$dur_mins ) = @_;
	my( $end_hrs,$end_mins,$end_time );
	( $end_hrs,$end_mins ) = split( /:/, $time );
	$end_mins += $dur_mins;
	if( $end_mins >= 60 ) { $end_hrs++; $end_mins -= 60; }
	$end_hrs += $dur_hrs;
	if( $end_hrs >= 24 ) { $end_hrs -= 24; }
	$end_time = sprintf( "%.2d", $end_hrs ).":".sprintf( "%.2d", $end_mins );
	return( $end_time );
}

# SUB NOTIFICATION #

# See comments in the framecal.pl file.  Note that for this implementation,
# there are some differences to account for the fact that there is only one
# type of notification used by this program, and that is a "Reminder", so
# the type argument is not used, and it simplifies the logic of generating
# the email message.

sub notification {
	my( $date,$time,$time_zone,$dur_hrs,$dur_mins,$notified,
	    $location,$subject,$body,$url,$title,$cat,$email,$name ) = @_;
	my( @notified, $notify_user, $msg_subject, $message, $type_word );
	my( $end_time,$temp_cat );
	@notified = split( /,/, $notified );
	$time_zone = ( $fcs::use_time_zones ? " - $time_zone" : "" );
	if( $dur_hrs || $dur_mins ) {
		$end_time = &get_end_time( $time, $dur_hrs, $dur_mins );
	}
	$temp_cat = $cat;
	$temp_cat =~ tr/_/ /;
	$body =~ s/<p>/\n\n/gio;
	$body =~ s/<br>/\n/gio;
	$msg_subject = "Event Reminder - $fcs::cal_name Calendar";
	$message = "This email is to remind you of the following event in ";
	$message .= "the $fcs::cal_name Calendar.\n($fcs::html_url)\n\n";
	$message .= "   Subject:  $subject\n";
	$message .= "   Contact:  $name ($email)\n";
	$message .= "Event Time:  $date";
	if( $end_time ) {
		$message .= " from $fcs::TIME{$time} to $fcs::TIME{$end_time}$time_zone\n";
	} else {
		$message .= " ( All Day )\n";
	}
	$message .= "  Location:  $location\n";
	$message .= "  Category:  $temp_cat\n" if( $temp_cat ne '' );
	$message .= "  Web Page:  $title ($url)\n" if( ($title ne '') && ($url ne '') );
	$message .= "\n$body\n";
	foreach $notify_user ( @notified ) {
		&mail::send_mail( $email, $notify_user, $msg_subject, $message );
	}
}